/*
 *   Copyright © 2009 Fredrik Höglund <fredrik@kde.org>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public License
 *   along with this library; see the file COPYING.LIB.  If not, write to
 *   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *   Boston, MA 02110-1301, USA.
 */

#include "strippedFolderView.h"

#include <KDesktopFile>
#include <KDirModel>
#include <KFileItemDelegate>
#include <kfilepreviewgenerator.h>
#include <Plasma/Theme>
#include <QGraphicsLinearLayout>

#include "dirlister.h"
#include "folderviewadapter.h"
#include "iconview.h"
#include "proxymodel.h"

StrippedFolderView::StrippedFolderView(QGraphicsWidget *parent, const KUrl &url, QSize size)
: QGraphicsWidget(parent), m_iconView(0),m_dirModel(0),m_model(0),m_url(url), m_size(size){
    if (m_size.isEmpty()){
        int s = KIconLoader::global()->currentSize(KIconLoader::Desktop);
        m_size.setHeight(s);
        m_size.setWidth(s);
    }
    DirLister *lister = new DirLister(this);
    lister->setDelayedMimeTypes(true);
    lister->setAutoErrorHandlingEnabled(false, 0);
    lister->openUrl(m_url);

    m_dirModel = new KDirModel(this);
    m_dirModel->setDropsAllowed(KDirModel::DropOnDirectory | KDirModel::DropOnLocalExecutable);
    m_dirModel->setDirLister(lister);

    m_model = new ProxyModel(this);
    m_model->setSourceModel(m_dirModel);
    m_model->setSortLocaleAware(true);
    m_model->setFilterCaseSensitivity(Qt::CaseInsensitive);
    m_model->setParseDesktopFiles(m_url.protocol() == "desktop");
    m_model->sort(KDirModel::Name, Qt::AscendingOrder);

    m_delegate = new KFileItemDelegate(this);
    m_selectionModel = new QItemSelectionModel(m_model, this);

    m_iconView = new IconView(this);
    m_iconView->setModel(m_model);
    m_iconView->setItemDelegate(m_delegate);
    m_iconView->setSelectionModel(m_selectionModel);
    m_iconView->setFont(Plasma::Theme::defaultTheme()->font(Plasma::Theme::DesktopFont));

    m_iconView->setPalette(QPalette(QPalette::Text, Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor)));
    m_iconView->setDrawShadows(true);
    m_iconView->setIconSize(iconSize());
    m_iconView->setGridSize(gridSize());
    m_iconView->setWordWrap(true);
    m_iconView->setIconsMoveable(false);

    connect(m_iconView, SIGNAL(activated(QModelIndex)), SLOT(activated(QModelIndex)));

    FolderViewAdapter *adapter = new FolderViewAdapter(m_iconView);
    m_previewGenerator = new KFilePreviewGenerator(adapter, m_model);
    m_previewGenerator->setPreviewShown(true);
    m_previewGenerator->setEnabledPlugins(QStringList() << "imagethumbnail" << "jpegthumbnail");

    QGraphicsLinearLayout *l = new QGraphicsLinearLayout();
	l->setOrientation(Qt::Vertical);
	l->addItem(m_iconView);
	setLayout(l);
}

StrippedFolderView::~StrippedFolderView()
{
}

QSize StrippedFolderView::iconSize() const
{
    return m_size;
}

QSize StrippedFolderView::gridSize() const
{
    const QFontMetrics fm(Plasma::Theme::defaultTheme()->font(Plasma::Theme::DesktopFont));
    const int textHeight = fm.lineSpacing() * 2; // ( 2 == numTextLines );
    QSize size = iconSize();
    size.rheight() = size.height() + textHeight + 16;
    size.rwidth() = qMax(size.width() * 2, fm.averageCharWidth() * 10);
    return size;
}

void StrippedFolderView::activated(const QModelIndex &index)
{
    const KFileItem item = m_model->itemForIndex(index);
    item.run();
}

#include "strippedFolderView.moc"

